# Generated by Django 4.2.13 on 2024-07-02 10:42

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        ("contenttypes", "0002_remove_content_type_name"),
    ]

    operations = [
        migrations.CreateModel(
            name="Relation",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("subj_object_id", models.PositiveIntegerField()),
                ("obj_object_id", models.PositiveIntegerField()),
                (
                    "obj_content_type",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="relation_obj_set",
                        to="contenttypes.contenttype",
                    ),
                ),
                (
                    "subj_content_type",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="relation_subj_set",
                        to="contenttypes.contenttype",
                    ),
                ),
            ],
            bases=(models.Model,),
        ),
    ]



import logging

from django.contrib.contenttypes.models import ContentType
from django.db.models.signals import post_delete, post_save
from django.dispatch import receiver

from apis_core.generic.signals import post_duplicate, post_merge_with
from apis_core.relations.models import Relation

logger = logging.getLogger(__name__)


@receiver(post_duplicate)
def copy_relations(sender, instance, duplicate, **kwargs):
    logger.info(f"Copying relations from {instance!r} to {duplicate!r}")
    content_type = ContentType.objects.get_for_model(instance)
    subj_rels = Relation.objects.filter(
        subj_content_type=content_type, subj_object_id=instance.id
    ).select_subclasses()
    obj_rels = Relation.objects.filter(
        obj_content_type=content_type, obj_object_id=instance.id
    ).select_subclasses()
    for rel in subj_rels:
        rel.pk = None
        rel.id = None
        rel.subj_object_id = duplicate.id
        rel.save()
    for rel in obj_rels:
        rel.pk = None
        rel.id = None
        rel.obj_object_id = duplicate.id
        rel.save()


@receiver(post_merge_with)
def merge_relations(sender, instance, entities, **kwargs):
    for ent in entities:
        logger.info(f"Merging relations from {ent!r} into {instance!r}")
        content_type = ContentType.objects.get_for_model(ent)
        Relation.objects.filter(
            subj_content_type=content_type, subj_object_id=ent.id
        ).update(subj_object_id=instance.id)
        Relation.objects.filter(
            obj_content_type=content_type, obj_object_id=ent.id
        ).update(obj_object_id=instance.id)


@receiver(post_delete)
def set_relations_null(sender, instance, using, origin, **kwargs):
    content_type = ContentType.objects.get_for_model(instance)
    object_id = instance.pk
    if isinstance(object_id, int):
        Relation.objects.filter(
            subj_content_type=content_type, subj_object_id=object_id
        ).update(subj_object_id=None)
        Relation.objects.filter(
            obj_content_type=content_type, obj_object_id=object_id
        ).update(obj_object_id=None)


@receiver(post_save)
def create_relations(sender, instance, created, raw, using, update_fields, **kwargs):
    """
    This signal looks at the `create_relations_to_uris` attribute of a model
    instance. The attribute should contain a dict mapping between a relation
    name and mapping between they key `obj` or `subj` and a list of URIs.
    The signal then tries to create relations between the instance and the
    subjects or objects listed in the relation dict mapping.
    An example for the dict mapping would be:
    "myapp.livesin" = {
        curies = ["https://example.org/123"],
        obj = "apis_ontology.place"
    }
    """

    # disable the handler during fixture loading
    if raw:
        return
    relations = getattr(instance, "create_relations_to_uris", {}).copy()
    for relation, details in relations.items():
        relation_model = ContentType.objects.get_by_natural_key(
            *relation.split(".")
        ).model_class()

        target = details.get("obj", None) or details.get("subj", None)
        target_content_type = ContentType.objects.get_by_natural_key(*target.split("."))
        related_model = target_content_type.model_class()

        for related_uri in details["curies"]:
            try:
                related_instance = related_model.import_from(uri=related_uri)
                if details.get("obj"):
                    relation_model.objects.create_between_instances(
                        instance, related_instance
                    )
                else:
                    relation_model.objects.create_between_instances(
                        related_instance, instance
                    )
            except Exception as e:
                logger.error(
                    "Could not create relation to %s due to %s", related_uri, e
                )
        del instance.create_relations_to_uris[relation]



import json

from django.contrib import messages
from django.contrib.auth import views as auth_views
from django.urls import reverse_lazy
from django.utils.translation import gettext_lazy as _
from drf_spectacular.utils import OpenApiParameter, extend_schema, inline_serializer
from rest_framework.permissions import IsAuthenticated
from rest_framework.response import Response
from rest_framework.views import APIView

from apis_core.utils.helpers import datadump_serializer


class Dumpdata(APIView):
    """
    provide an API endpoint that outputs the datadump of an APIS installation

    this is a bit of a hack, becaus we first use the Django JSON serializer to
    serialize the data using natural keys, then we use json.loads to so we can
    output it as an API reponse.
    so basically: serialize -> deserialize -> serialize
    """

    permission_classes = [IsAuthenticated]

    @extend_schema(
        parameters=[OpenApiParameter(name="app_labels", type=str, many=True)],
        responses={200: inline_serializer(name="DumpDataResponse", fields={})},
    )
    def get(self, request, *args, **kwargs):
        params = request.query_params.dict()
        app_labels = params.pop("app_labels", [])
        if app_labels:
            app_labels = app_labels.split(",")
        return Response(json.loads(datadump_serializer(app_labels, "json")))


class PasswordChangeView(auth_views.PasswordChangeView):
    template_name = "profile/password-change.html"
    success_url = reverse_lazy("apis_core:password-change")

    def form_valid(self, form):
        ret = super().form_valid(form)
        messages.success(
            self.request,
            _("Password for %(user)s changed.").format(user=self.request.user),
        )
        return ret



from django.urls import path

from apis_core.history import views

app_name = "history"


urlpatterns = [
    path(
        "<contenttype:contenttype>/<int:pk>/history",
        views.HistoryView.as_view(),
        name="history",
    ),
    path(
        "<contenttype:contenttype>/<int:pk>/reset",
        views.HistoryReset.as_view(),
        name="reset",
    ),
]



# Generated by Django 5.2.2 on 2025-07-01 12:49

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("contenttypes", "0002_remove_content_type_name"),
        ("relations", "0002_alter_relation_obj_object_id_and_more"),
    ]

    operations = [
        migrations.AddIndex(
            model_name="relation",
            index=models.Index(
                fields=["subj_content_type"], name="relations_r_subj_content_type"
            ),
        ),
        migrations.AddIndex(
            model_name="relation",
            index=models.Index(
                fields=["subj_object_id"], name="relations_r_subj_object_id"
            ),
        ),
        migrations.AddIndex(
            model_name="relation",
            index=models.Index(
                fields=["obj_content_type"], name="relations_r_obj_content_type"
            ),
        ),
        migrations.AddIndex(
            model_name="relation",
            index=models.Index(
                fields=["obj_object_id"], name="relations_r_obj_object_id"
            ),
        ),
        migrations.AddIndex(
            model_name="relation",
            index=models.Index(
                fields=["subj_content_type", "subj_object_id"],
                name="relations_r_subj_c_t_o_i",
            ),
        ),
        migrations.AddIndex(
            model_name="relation",
            index=models.Index(
                fields=["obj_content_type", "obj_object_id"],
                name="relations_r_obj_c_t_o_i",
            ),
        ),
    ]



# Generated by Django 5.2.5 on 2025-08-25 05:53

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("sample_project", "0009_versionprofession"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="group",
            options={
                "ordering": ["label"],
                "verbose_name": "group",
                "verbose_name_plural": "groups",
            },
        ),
        migrations.AlterModelOptions(
            name="person",
            options={
                "ordering": ["surname", "forename"],
                "verbose_name": "person",
                "verbose_name_plural": "persons",
            },
        ),
        migrations.AlterModelOptions(
            name="place",
            options={
                "ordering": ["label"],
                "verbose_name": "place",
                "verbose_name_plural": "places",
            },
        ),
    ]



import os

from apis_core.utils.autocomplete import (
    ExternalAutocomplete,
    LobidAutocompleteAdapter,
    TypeSenseAutocompleteAdapter,
)


class GroupExternalAutocomplete(ExternalAutocomplete):
    adapters = [
        LobidAutocompleteAdapter(
            params={
                "filter": "type:CorporateBody",
                "format": "json:preferredName,geographicAreaCode,dateOfEstablishment,broaderTermInstantial",
            },
            data_mapping={"label": "label"},
        ),
    ]


class PlaceExternalAutocomplete(ExternalAutocomplete):
    adapters = [
        TypeSenseAutocompleteAdapter(
            collections=[
                "prosnet-wikidata-place-index",
                "prosnet-geonames-place-index",
            ],
            token=os.getenv("TYPESENSE_TOKEN", None),
            server=os.getenv("TYPESENSE_SERVER", None),
            data_mapping={"label": ["document", "label"]},
        ),
        LobidAutocompleteAdapter(
            params={
                "filter": "type:PlaceOrGeographicName",
                "format": "json:preferredName",
            },
            data_mapping={"label": "label"},
        ),
    ]


class PersonExternalAutocomplete(ExternalAutocomplete):
    adapters = [
        TypeSenseAutocompleteAdapter(
            collections="prosnet-wikidata-person-index",
            token=os.getenv("TYPESENSE_TOKEN", None),
            server=os.getenv("TYPESENSE_SERVER", None),
        ),
        LobidAutocompleteAdapter(
            params={
                "filter": "type:Person",
                "format": "json:preferredName,professionOrOccupation",
            },
            data_mapping={"surname": "label"},
        ),
    ]



from rdflib import Namespace

from apis_core.utils.settings import apis_base_uri

CRM = Namespace("http://www.cidoc-crm.org/cidoc-crm/")

APPELLATION = Namespace(apis_base_uri() + "/appellation/")
ATTRIBUTES = Namespace(apis_base_uri() + "/attributes/")



import difflib

from django.apps import apps
from django.core import serializers
from django.db import DEFAULT_DB_ALIAS, router


def datadump_get_objects(models: list = [], *args, **kwargs):
    for model in models:
        if not model._meta.proxy and router.allow_migrate_model(
            DEFAULT_DB_ALIAS, model
        ):
            objects = model._default_manager
            queryset = objects.using(DEFAULT_DB_ALIAS).order_by(model._meta.pk.name)
            yield from queryset.iterator()


def datadump_get_queryset(additional_app_labels: list = []):
    """
    This method is loosely based on the `dumpdata` admin command.
    It iterates throug the relevant app models and exports them using
    a serializer and natural foreign keys.
    Data exported this way can be reimported into a newly created Django APIS app
    """

    # get all APIS apps and all APIS models
    apis_app_labels = ["apis_relations", "apis_metainfo"]
    apis_app_models = [
        model for model in apps.get_models() if model._meta.app_label in apis_app_labels
    ]

    # create a list of app labels we want to iterate
    # this allows to extend the apps via the ?app_labels= parameter
    app_labels = set(apis_app_labels)
    app_labels |= set(additional_app_labels)

    # look for models that inherit from APIS models and add their
    # app label to app_labels
    for model in apps.get_models():
        if any(map(lambda x: issubclass(model, x), apis_app_models)):
            app_labels.add(model._meta.app_label)

    # now go through all app labels
    app_list = {}
    for app_label in app_labels:
        app_config = apps.get_app_config(app_label)
        app_list[app_config] = None

    models = serializers.sort_dependencies(app_list.items(), allow_cycles=True)

    yield from datadump_get_objects(models)


def datadump_serializer(additional_app_labels: list = [], serialier_format="json"):
    return serializers.serialize(
        serialier_format,
        datadump_get_queryset(additional_app_labels),
        use_natural_foreign_keys=True,
    )


def get_html_diff(a, b, show_a=True, show_b=True, shorten=0):
    """
    Create an colorized html represenation of the difference of two values a and b
    If `show_a` is True, colorize deletions in `a`
    If `show_b` is True, colorize insertions in `b`
    The value of `shorten` defines if long parts of strings that contains no change should be shortened
    """

    def style_remove(text):
        return f"<span class='diff-remove'>{text}</span>"

    def style_insert(text):
        return f"<span class='diff-insert'>{text}</span>"

    nones = ["", None]
    if a in nones and b in nones:
        result = ""
    elif a in nones:
        result = style_insert(b) if show_b else ""
    elif b in nones:
        result = style_remove(a) if show_a else ""
    else:
        result = ""
        a = str(a)
        b = str(b)
        codes = difflib.SequenceMatcher(None, a, b).get_opcodes()
        for opcode, a_start, a_end, b_start, b_end in codes:
            match opcode:
                case "equal":
                    equal = a[a_start:a_end]
                    if shorten and len(equal) > shorten:
                        equal = equal[:5] + " ... " + equal[-10:]
                    result += equal
                case "delete":
                    if show_a:
                        result += style_remove(a[a_start:a_end])
                case "insert":
                    if show_b:
                        result += style_insert(b[b_start:b_end])
                case "replace":
                    if show_b:
                        result += style_insert(b[b_start:b_end])
                    if show_a:
                        result += style_remove(a[a_start:a_end])
    return result


def construct_lookup(value: str) -> tuple[str, str]:
    """
    Helper method to parse input values and construct field lookups
    (https://docs.djangoproject.com/en/4.2/ref/models/querysets/#field-lookups)
    Parses user input for wildcards and returns a tuple containing the
    interpreted django lookup string and the trimmed value
    E.g.

    - ``example`` -> ``('__icontains', 'example')``
    - ``*example*`` -> ``('__icontains', 'example')``
    - ``*example`` -> ``('__iendswith', 'example')``
    - ``example*``-> ``('__istartswith', 'example')``
    - ``"example"`` -> ``('__iexact', 'example')``

    :param str value: text to be parsed for ``*``
    :return: a tuple containing the lookup type and the value without modifiers
    """

    if value.startswith("*") and not value.endswith("*"):
        value = value[1:]
        return "__iendswith", value

    elif not value.startswith("*") and value.endswith("*"):
        value = value[:-1]
        return "__istartswith", value

    elif value.startswith('"') and value.endswith('"'):
        value = value[1:-1]
        return "__iexact", value

    else:
        if value.startswith("*") and value.endswith("*"):
            value = value[1:-1]
        return "__icontains", value


def flatten_if_single(value: list):
    if len(value) == 1:
        return value[0]
    return value



import functools
from pathlib import Path

from django.apps import apps
from django.contrib.contenttypes.models import ContentType

from apis_core.apis_entities.models import AbstractEntity


def get_entity_classes():
    return list(filter(lambda x: issubclass(x, AbstractEntity), apps.get_models()))


def get_entity_content_types():
    return [ContentType.objects.get_for_model(model) for model in get_entity_classes()]


@functools.lru_cache
def get_feature_codes():
    try:
        featurecode_list = [(None, "---")]
        self = Path(__file__)
        featurecodefile = self.parent / "featureCodes_en.txt"
        featurecodes = featurecodefile.read_text()
        featurecodes = [
            line.split("\t")
            for line in featurecodes.split("\n")
            if len(line.split("\t")) == 3
        ]
        featurecode_list.extend([(el[0], el[0] + ": " + el[1]) for el in featurecodes])
    except Exception:
        featurecode_list = [(None, "None: could not read list of feature codes")]
    return featurecode_list



# Generated by Django 5.1.5 on 2025-04-17 09:17

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        (
            "sample_project",
            "0006_alter_group_label_alter_person_date_of_birth_and_more",
        ),
    ]

    operations = [
        migrations.AlterField(
            model_name="person",
            name="forename",
            field=models.CharField(
                blank=True, default="", max_length=4096, verbose_name="forename"
            ),
        ),
        migrations.AlterField(
            model_name="versionperson",
            name="forename",
            field=models.CharField(
                blank=True, default="", max_length=4096, verbose_name="forename"
            ),
        ),
    ]



from django.db import models
from django.utils.encoding import force_str
from django.utils.translation import gettext_lazy as _

from apis_core.generic.utils.rdf_namespace import CRM
from apis_core.utils.rdf import load_uri_using_path

#########################
# Abstract base classes #
#########################


# These abstract base classes are named after
# CIDOC CRM entities, but we are *NOT*(!)
# trying to implement CIDOC CRM in Django.


class E21_Person(models.Model):
    forename = models.CharField(
        blank=True, default="", max_length=4096, verbose_name=_("forename")
    )
    surname = models.CharField(
        blank=True, default="", max_length=4096, verbose_name=_("surname")
    )
    gender = models.CharField(
        blank=True, default="", max_length=4096, verbose_name=_("gender")
    )
    date_of_birth = models.DateField(
        blank=True, null=True, verbose_name=_("date of birth")
    )
    date_of_death = models.DateField(
        blank=True, null=True, verbose_name=_("date of death")
    )

    class Meta:
        abstract = True
        verbose_name = _("person")
        verbose_name_plural = _("persons")
        ordering = ["surname", "forename"]

    def __str__(self):
        if self.forename and self.surname:
            return f"{self.forename} {self.surname}"
        return self.forename or self.surname or force_str(_("No name"))

    import_definitions = {
        "https://d-nb.info/*|/.*.rdf": lambda x: load_uri_using_path(
            x, "triple_configs/E21_PersonFromDNB.toml"
        ),
        "http://www.wikidata.org/*|/.*.rdf": lambda x: load_uri_using_path(
            x, "triple_configs/E21_PersonFromWikidata.toml"
        ),
    }

    @classmethod
    def get_rdf_types(cls):
        return [CRM.E21_Person]


class E53_Place(models.Model):
    """
    The feature_code field refers to the geonames feature codes, as
    listed on https://www.geonames.org/export/codes.html
    """

    label = models.CharField(
        blank=True, default="", max_length=4096, verbose_name=_("label")
    )
    latitude = models.FloatField(blank=True, null=True, verbose_name=_("latitude"))
    longitude = models.FloatField(blank=True, null=True, verbose_name=_("longitude"))
    feature_code = models.CharField(
        blank=True,
        default="",
        max_length=16,
        verbose_name=_("feature code"),
        help_text='<a href="https://www.geonames.org/export/codes.html" target="_blank">Geonames Feature Code List</a>',
    )

    class Meta:
        abstract = True
        verbose_name = _("place")
        verbose_name_plural = _("places")
        ordering = ["label"]

    def __str__(self):
        return self.label or force_str(_("No label"))

    import_definitions = {
        "https://d-nb.info/*|/.*.rdf": lambda x: load_uri_using_path(
            x, "triple_configs/E53_PlaceFromDNB.toml"
        ),
        "https://sws.geonames.org/*|/.*.rdf*": lambda x: load_uri_using_path(
            x, "triple_configs/E53_PlaceFromGeonames.toml"
        ),
        "http://www.wikidata.org/*|/.*.rdf": lambda x: load_uri_using_path(
            x, "triple_configs/E53_PlaceFromWikidata.toml"
        ),
    }

    @classmethod
    def get_rdf_types(cls):
        return [CRM.E53_Place]

    @classmethod
    def create_from_string(cls, string):
        return cls.objects.create(label=string)


class E74_Group(models.Model):
    label = models.CharField(
        blank=True, default="", max_length=4096, verbose_name=_("label")
    )

    class Meta:
        abstract = True
        verbose_name = _("group")
        verbose_name_plural = _("groups")
        ordering = ["label"]

    def __str__(self):
        return self.label or force_str(_("No label"))

    import_definitions = {
        "https://d-nb.info/*|/.*.rdf": lambda x: load_uri_using_path(
            x, "triple_configs/E74_GroupFromDNB.toml"
        ),
        "http://www.wikidata.org/*|/.*.rdf": lambda x: load_uri_using_path(
            x, "triple_configs/E74_GroupFromWikidata.toml"
        ),
    }

    @classmethod
    def get_rdf_types(cls):
        return [CRM.E74_Group]

    @classmethod
    def create_from_string(cls, string):
        return cls.objects.create(label=string)


class SimpleLabelModel(models.Model):
    label = models.CharField(
        blank=True, default="", max_length=4096, verbose_name=_("label")
    )

    class Meta:
        abstract = True
        ordering = ["label"]

    def __str__(self):
        return self.label or force_str(_("No label"))

    @classmethod
    def create_from_string(cls, string):
        return cls.objects.create(label=string)



from django.db import models


class Person(models.Model):
    first_name = models.CharField()
    last_name = models.CharField()

    def __str__(self):
        return f"{self.first_name} {self.last_name}"

    def get_default_uri(self):
        return f"https://www.example.org/id/{self.pk}"


class Dummy(models.Model):
    pass



import pytest
from django.contrib.contenttypes.models import ContentType
from faker import Faker
from pytest_django.asserts import assertContains, assertTemplateUsed

from apis_core.uris.models import Uri
from apis_core.uris.tests.models import Dummy, Person

fake = Faker()


@pytest.mark.django_db
class TestUris(object):
    @pytest.fixture
    def setup(self):
        # Create a couple of person objects
        for _ in range(0, fake.random_int(5, 20)):
            Person.objects.create(
                first_name=fake.first_name(), last_name=fake.last_name()
            )

    def test_uris_count(self, setup):
        """Check if the correct number of Uris was created"""
        assert Uri.objects.count() == Person.objects.count()

    def test_uris_list_view_status(self, setup, admin_client):
        """Check if the uris list view works"""
        response = admin_client.get("/uris.uri/")
        assert response.status_code == 200

    def test_uris_list_view_content(self, setup, admin_client):
        """Check if the uris list view works"""
        response = admin_client.get("/uris.uri/")
        count = Person.objects.count()
        # check for the correct result count in the response content
        assertContains(response, f"{count} results.")
        # check if all the persons are listed
        for p in Person.objects.all():
            assertContains(response, str(p))

    def test_delete_view_template_override(self, setup, admin_client):
        """Check that the delete confirm view uses the right template"""
        response = admin_client.get("/uris.uri/delete/1")
        uri = Uri.objects.get(pk=1)
        assertTemplateUsed(response, "uris/uri_confirm_delete.html")
        assertContains(response, uri.uri)
        assertContains(response, str(uri.content_object))

    def test_no_uris_for_dummy(self):
        """Check that we don't create uris for dummy objects"""
        for _ in range(0, 5):
            Dummy.objects.create()
        content_type = ContentType.objects.get_for_model(Dummy)
        assert Uri.objects.filter(content_type=content_type).count() == 0

    def test_custom_uri(self):
        d = Dummy.objects.create()
        uri = "http://example.org"
        d._uris = [uri]
        d.save()
        assert Uri.objects.filter(uri=uri).exists()
        assert Uri.objects.get(uri=uri).content_object == d



# Generated by Django 5.2.4 on 2025-08-05 09:28

import django.db.models.deletion
from django.db import migrations, models

import apis_core.generic.abc


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        ("contenttypes", "0002_remove_content_type_name"),
    ]

    operations = [
        migrations.SeparateDatabaseAndState(
            state_operations=[
                migrations.CreateModel(
                    name="Uri",
                    fields=[
                        (
                            "id",
                            models.AutoField(
                                auto_created=True,
                                primary_key=True,
                                serialize=False,
                                verbose_name="ID",
                            ),
                        ),
                        (
                            "uri",
                            models.URLField(
                                blank=True, max_length=255, null=True, unique=True
                            ),
                        ),
                        ("object_id", models.PositiveIntegerField(null=True)),
                        (
                            "content_type",
                            models.ForeignKey(
                                null=True,
                                on_delete=django.db.models.deletion.CASCADE,
                                to="contenttypes.contenttype",
                            ),
                        ),
                    ],
                    bases=(apis_core.generic.abc.GenericModel, models.Model),
                ),
            ],
            database_operations=[],
        )
    ]



# Generated by Django 4.1.11 on 2023-10-11 10:14

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("contenttypes", "0002_remove_content_type_name"),
        ("apis_metainfo", "0007_delete_source"),
    ]

    operations = [
        migrations.AlterField(
            model_name="rootobject",
            name="self_contenttype",
            field=models.ForeignKey(
                blank=True,
                editable=False,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                to="contenttypes.contenttype",
            ),
        ),
    ]



import django_tables2 as tables
from django.apps import apps

from apis_core.generic.tables import ActionColumn, CustomTemplateColumn, ViewColumn


class DescriptionColumnHistory(CustomTemplateColumn):
    """
    A column showing a model description
    """

    template_name = "history/columns/description.html"
    orderable = False


class OriginalIDColumn(CustomTemplateColumn):
    """
    A column showing the original id of a model instance
    """

    template_name = "history/columns/original_id.html"
    orderable = False
    verbose_name = "Original ID"


class ResetColumn(ActionColumn):
    """
    A column showing a reset button
    """

    template_name = "history/columns/reset.html"
    permission = "change"


class APISHistoryTableBaseTable(tables.Table):
    history_id = tables.Column(verbose_name="ID")
    original_id = OriginalIDColumn()
    desc = DescriptionColumnHistory()
    view = ViewColumn()

    class Meta:
        fields = ["history_id", "desc", "most_recent", "view"]


class HistoryGenericTable(tables.Table):
    model = tables.Column(empty_values=())
    fields_changed = tables.Column(empty_values=())
    instance = tables.Column(linkify=lambda record: record.get_absolute_url())
    fields_changed = tables.TemplateColumn(
        template_name="history/columns/fields_changed.html"
    )
    reset = ResetColumn()

    class Meta:
        fields = [
            "model",
            "instance",
            "fields_changed",
            "history_type",
            "history_date",
            "history_user",
        ]

    def render_model(self, record):
        return record.instance.__class__.__name__

    def __init__(self, *args, **kwargs):
        print(kwargs)
        if apps.is_installed("apis_core.collections"):
            from apis_core.collections.columns import CollectionsColumn

            kwargs["extra_columns"] = [("collections", CollectionsColumn())]
        super().__init__(*args, **kwargs)



# Generated by Django 5.1.7 on 2025-03-27 08:14

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("sample_project", "0005_place_feature_code_versionplace_feature_code"),
    ]

    operations = [
        migrations.AlterField(
            model_name="group",
            name="label",
            field=models.CharField(
                blank=True, default="", max_length=4096, verbose_name="label"
            ),
        ),
        migrations.AlterField(
            model_name="person",
            name="date_of_birth",
            field=models.DateField(blank=True, null=True, verbose_name="date of birth"),
        ),
        migrations.AlterField(
            model_name="person",
            name="date_of_death",
            field=models.DateField(blank=True, null=True, verbose_name="date of death"),
        ),
        migrations.AlterField(
            model_name="person",
            name="forename",
            field=models.CharField(
                blank=True, default="", max_length=4096, verbose_name="forname"
            ),
        ),
        migrations.AlterField(
            model_name="person",
            name="gender",
            field=models.CharField(
                blank=True, default="", max_length=4096, verbose_name="gender"
            ),
        ),
        migrations.AlterField(
            model_name="person",
            name="surname",
            field=models.CharField(
                blank=True, default="", max_length=4096, verbose_name="surname"
            ),
        ),
        migrations.AlterField(
            model_name="place",
            name="feature_code",
            field=models.CharField(
                blank=True,
                default="",
                help_text='<a href="https://www.geonames.org/export/codes.html">Geonames Feature Code List</a>',
                max_length=16,
                verbose_name="feature code",
            ),
        ),
        migrations.AlterField(
            model_name="place",
            name="label",
            field=models.CharField(
                blank=True, default="", max_length=4096, verbose_name="label"
            ),
        ),
        migrations.AlterField(
            model_name="place",
            name="latitude",
            field=models.FloatField(blank=True, null=True, verbose_name="latitude"),
        ),
        migrations.AlterField(
            model_name="place",
            name="longitude",
            field=models.FloatField(blank=True, null=True, verbose_name="longitude"),
        ),
        migrations.AlterField(
            model_name="versiongroup",
            name="label",
            field=models.CharField(
                blank=True, default="", max_length=4096, verbose_name="label"
            ),
        ),
        migrations.AlterField(
            model_name="versionperson",
            name="date_of_birth",
            field=models.DateField(blank=True, null=True, verbose_name="date of birth"),
        ),
        migrations.AlterField(
            model_name="versionperson",
            name="date_of_death",
            field=models.DateField(blank=True, null=True, verbose_name="date of death"),
        ),
        migrations.AlterField(
            model_name="versionperson",
            name="forename",
            field=models.CharField(
                blank=True, default="", max_length=4096, verbose_name="forname"
            ),
        ),
        migrations.AlterField(
            model_name="versionperson",
            name="gender",
            field=models.CharField(
                blank=True, default="", max_length=4096, verbose_name="gender"
            ),
        ),
        migrations.AlterField(
            model_name="versionperson",
            name="surname",
            field=models.CharField(
                blank=True, default="", max_length=4096, verbose_name="surname"
            ),
        ),
        migrations.AlterField(
            model_name="versionplace",
            name="feature_code",
            field=models.CharField(
                blank=True,
                default="",
                help_text='<a href="https://www.geonames.org/export/codes.html">Geonames Feature Code List</a>',
                max_length=16,
                verbose_name="feature code",
            ),
        ),
        migrations.AlterField(
            model_name="versionplace",
            name="label",
            field=models.CharField(
                blank=True, default="", max_length=4096, verbose_name="label"
            ),
        ),
        migrations.AlterField(
            model_name="versionplace",
            name="latitude",
            field=models.FloatField(blank=True, null=True, verbose_name="latitude"),
        ),
        migrations.AlterField(
            model_name="versionplace",
            name="longitude",
            field=models.FloatField(blank=True, null=True, verbose_name="longitude"),
        ),
    ]



from pathlib import Path

from django.apps import apps
from django.contrib.contenttypes.models import ContentType
from django.core import serializers
from django.core.management.base import BaseCommand
from rest_framework.relations import RelatedField
from rest_framework.renderers import JSONRenderer

from apis_core.generic.abc import GenericModel
from apis_core.generic.helpers import first_member_match, module_paths
from apis_core.generic.renderers import CidocTTLRenderer
from apis_core.generic.serializers import (
    GenericHyperlinkedModelSerializer,
    serializer_factory,
)


class NaturalKeyRelatedField(RelatedField):
    def to_representation(self, value):
        content_type = ContentType.objects.get_for_model(value)
        return {
            "model": f"{content_type.app_label}.{content_type.model}",
            "pk": value.pk,
        }


class Command(BaseCommand):
    help = "Export data"

    def add_arguments(self, parser):
        parser.add_argument("path", type=Path)
        parser.add_argument(
            "--serializers",
            nargs="+",
            type=str,
            choices=["json", "jsonl", "drf_json", "drf_jsonl", "drf_cidoc_ttl"],
            default=["json"],
        )

    def get_serializer_class(self, model, prefix=""):
        serializer_class_modules = module_paths(
            model, path="serializers", suffix=f"{prefix}Serializer"
        )
        serializer_class = first_member_match(
            serializer_class_modules, GenericHyperlinkedModelSerializer
        )
        # We force the found serializer to use our NaturalKeyRelatedField
        # instead of the HyperlinkedRelatedField, which gives us a URI
        serializer_class.serializer_related_field = NaturalKeyRelatedField
        # We fore the url_field name to be `id`, which is a hack - the
        # HyperlinkedModelSerializer seems to replace the "id" with a URI,
        # but if we set the `uri_field_name` to `id` it gets overwritten
        # wit the original value again.
        serializer_class.url_field_name = "id"
        return serializer_factory(model, serializer=serializer_class)

    def serialize_json(self, model, content_type_str):
        folder = self.output / "json" / content_type_str
        folder.mkdir(parents=True, exist_ok=True)
        serializer = serializers.get_serializer("json")
        for instance in model.objects.all():
            data = serializer().serialize(
                [
                    instance,
                ],
                indent=2,
                use_natural_foreign_keys=True,
                use_natural_primary_keys=True,
            )
            file = folder / f"{instance.pk:06}.json"
            file.write_text(data)

    def serialize_jsonl(self, model, content_type_str):
        folder = self.output / "jsonl"
        folder.mkdir(parents=True, exist_ok=True)

        serializer = serializers.get_serializer("jsonl")
        data = serializer().serialize(
            model.objects.all(),
            indent=2,
            use_natural_foreign_keys=True,
            use_natural_primary_keys=True,
        )
        file = folder / f"{content_type_str}.jsonl"
        file.write_text(data)

    def serialize_drf_json(self, model, content_type_str):
        folder = self.output / "drf_json" / content_type_str
        folder.mkdir(parents=True, exist_ok=True)

        Serializer = self.get_serializer_class(model)

        for instance in model.objects.all():
            data = Serializer(instance, context={"request": None}).data
            file = folder / f"{instance.pk:06}.json"
            file.write_text(
                JSONRenderer()
                .render(data, accepted_media_type="application/json; indent=2")
                .decode()
            )

    def serialize_drf_jsonl(self, model, content_type_str):
        folder = self.output / "drf_jsonl"
        folder.mkdir(parents=True, exist_ok=True)

        Serializer = self.get_serializer_class(model)

        lines = []
        for instance in model.objects.all():
            data = Serializer(instance, context={"request": None}).data
            lines.append(JSONRenderer().render(data).decode())
        file = folder / f"{content_type_str}.jsonl"
        file.write_text("\n".join(lines))

    def serialize_drf_cidoc_ttl(self, model, content_type_str):
        folder = self.output / "drf_rdf"
        folder.mkdir(parents=True, exist_ok=True)

        Serializer = self.get_serializer_class(model, prefix="Cidoc")

        data = {
            "results": Serializer(
                model.objects.all(), context={"request": None}, many=True
            ).data
        }
        file = folder / f"{content_type_str}.cidoc.ttl"
        file.write_text(CidocTTLRenderer().render(data))

    def handle(self, *args, **options):
        self.output = options["path"]
        self.output.mkdir(parents=True, exist_ok=True)

        for model in apps.get_models():
            if issubclass(model, GenericModel):
                content_type = ContentType.objects.get_for_model(model)
                content_type_str = f"{content_type.app_label}.{content_type.model}"
                print(
                    f"Serializing {model} instances (content type: {content_type_str})"
                )

                for serializer in options["serializers"]:
                    getattr(self, f"serialize_{serializer}")(model, content_type_str)



# SPDX-FileCopyrightText: 2025 Birger Schacht
# SPDX-License-Identifier: MIT

import logging
import tomllib
from collections import defaultdict
from pathlib import Path

from AcdhArcheAssets.uri_norm_rules import get_normalized_uri
from django.template.exceptions import TemplateDoesNotExist
from django.template.loader import render_to_string
from django.template.utils import get_app_template_dirs
from rdflib import RDF, BNode, Graph, URIRef
from rdflib.exceptions import ParserError

logger = logging.getLogger(__name__)


def resolve(obj, graph):
    """
    Look at the value of object and return the parsed
    value. If the value starts and ens with angle brackets,
    we interpret it as and transform it to an URI.
    If the value is simple text we interpret it as an curie
    and we expand it using the graphs namespace manager.
    Otherwise we simply return the value
    """
    if isinstance(obj, str):
        if obj.startswith("<") and obj.endswith(">"):
            return URIRef(obj[1:-1])
        return graph.namespace_manager.expand_curie(obj)
    if isinstance(obj, bool) and obj is True:
        return None
    return obj


def load_path(path: str | Path) -> dict:
    """
    Load a tomlfile either from a path or from the directory
    `triple_configs` in any of the app directories.
    """
    try:
        return tomllib.loads(render_to_string(path))
    except TemplateDoesNotExist:
        logger.debug("Tried to load template %s but it does not exist", path)
    if isinstance(path, str):
        files = [
            directory / path for directory in get_app_template_dirs("triple_configs")
        ]
        files = list(filter(lambda file: file.exists(), files))
        if files:
            path = files[0]
        else:
            raise ValueError(f"Could not find {path}")
    return tomllib.loads(Path(path).read_text())


def graph_matches_config(graph: Graph, configfile: Path) -> dict:
    """
    Check if a file contains a config that matches this
    graph and if so, return the config as dict. Otherwise
    return False
    """
    config = load_path(configfile)
    for _filter in config.get("filters", [{None: None}]):
        try:
            triples = []
            for predicate, obj in _filter.items():
                triples.append((None, resolve(predicate, graph), resolve(obj, graph)))
            triples = [triple in graph for triple in triples]
            if all(triples):
                logger.debug("Using %s for parsing graph", configfile)
                return config
        except ValueError as e:
            logger.debug("Filter %s does not match: %s", _filter, e)
    return {}


def build_sparql_query(curie: str) -> str:
    """
    Build a SPARQL query with language preferences.

    Args:
        curie: predicate to filter on as defined in the toml.
                needs to include the predicate and optionally
                a lang tag to filter for separated with a comma.
                Eg "wdt:P122,en".

    Returns:
        A SPARQL query string
    """
    if curie.lower().strip().startswith(("select", "prefix")):
        return curie
    lang_tag = ""
    if "," in curie:
        curie, lang_tag = curie.split(",", 1)
        lang_tag = f'FILTER LANGMATCHES(LANG(?object), "{lang_tag}")'
    query = f"""
            SELECT ?object 
            WHERE {{ 
                ?subject {curie} ?object {lang_tag}
            }}
        """

    logger.debug("Generated SPARQL query: %s", query)
    return query


def get_value_graph(graph: Graph, curies: str | list[str]) -> list:
    values = []
    if isinstance(curies, str):
        curies = [curies]
    for curie in curies:
        results = graph.query(build_sparql_query(curie))
        objects = [result[0] for result in results]
        for obj in objects:
            if isinstance(obj, BNode):
                values.extend(
                    [
                        str(value)
                        for value in graph.objects(subject=obj)
                        if value != RDF.Seq
                    ]
                )
            else:
                values.append(str(obj))
    return list(dict.fromkeys(values))


def load_uri_using_path(uri, configfile: Path) -> dict:
    uri = get_normalized_uri(uri)
    graph = Graph()
    # workaround for a bug in d-nb: with the default list of accept
    # headers of rdflib, d-nb sometimes returns json-ld and sometimes turtle
    # with json-ld, rdflib has problems finding the namespaces
    format = "turtle" if uri.startswith("https://d-nb.info/gnd/") else None
    try:
        graph.parse(uri, format=format)
    except ParserError as e:
        logger.info(e)

    if config := graph_matches_config(graph, configfile):
        result = defaultdict(list)
        result["relations"] = defaultdict(list)
        for attribute, curies in config.get("attributes", {}).items():
            values = get_value_graph(graph, curies)
            result[attribute].extend(values)
        for relation, details in config.get("relations", {}).items():
            details["curies"] = get_value_graph(graph, details.get("curies", []))
            result["relations"][relation] = details
        return dict(result)
    return None



import base64
import json
import logging
from typing import Tuple
from urllib.parse import parse_qs, urlencode, urlparse, urlunparse

from AcdhArcheAssets.uri_norm_rules import get_normalized_uri

logger = logging.getLogger(__name__)


def get_autocomplete_data_and_normalized_uri(
    uri: str, key: str = "anero_ac_data"
) -> Tuple[dict, str]:
    urlparts = urlparse(uri)
    query = urlparts.query
    params = parse_qs(query)
    data = params.pop(key, {})
    if data:
        try:
            data = json.loads(base64.b64decode(data[0]))
        except Exception as e:
            logger.debug("Could not decode anero_ac_data: %s", e)
            data = {}
    urlparts = urlparts._replace(query=urlencode(params, doseq=True))
    uri = get_normalized_uri(urlunparse(urlparts))
    return data, uri



from django.db import models
from model_utils.managers import InheritanceManager

from apis_core.generic.abc import GenericModel


class RootObject(GenericModel, models.Model):
    """
    The very root thing that can exist in a given ontology. Several classes inherit from it.
    By having one overarching super class we gain the advantage of unique identifiers.
    """

    objects = models.Manager()
    objects_inheritance = InheritanceManager()



# Generated by Django 3.1.14 on 2023-03-10 08:04

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("apis_metainfo", "0003_auto_20220218_1530"),
    ]

    operations = [
        migrations.RenameField(
            model_name="rootobject",
            old_name="self_content_type",
            new_name="self_contenttype",
        ),
    ]
